#include <stdlib.h>
#include "petscmat.h"
#include "anamod.h"

/*
 * Test for attached quantities
 */
static PetscErrorCode computetwo(Mat A)
{
  int id; PetscBool has; PetscErrorCode ierr;
  PetscFunctionBegin;

  ierr = GetDataID("all","one",&id,&has); CHKERRQ(ierr);
  if (!has) SETERRQ(PETSC_COMM_SELF,1,"problem one");
  ierr = PetscObjectComposedDataSetInt((PetscObject)A,id,5); CHKERRQ(ierr);

  ierr = GetDataID("all","two",&id,&has); CHKERRQ(ierr);
  if (!has) SETERRQ(PETSC_COMM_SELF,1,"problem two");
  ierr = PetscObjectComposedDataSetInt((PetscObject)A,id,6); CHKERRQ(ierr);

  PetscFunctionReturn(0);
}

int main(int argc,char **argv)
{
  Mat A; AnalysisItem result;
  PetscBool flg; PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);

  ierr = MatCreateSeqAIJ(MPI_COMM_SELF,1,1,0,0,&A); CHKERRQ(ierr);
  ierr = RegisterModule
    ("all","one",ANALYSISINTEGER,NULL); CHKERRQ(ierr);
  ierr = RegisterModule
    ("all","two",ANALYSISINTEGER,NULL); CHKERRQ(ierr);

  ierr = computetwo(A); CHKERRQ(ierr);

  ierr = HasQuantity
    ((AnaModNumericalProblem)A,"all","one",&flg); CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity one missing");
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"all","one",&result,NULL,&flg); CHKERRQ(ierr);
  if (flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity one strangely present");
  ierr = RetrieveQuantity
    ((AnaModNumericalProblem)A,"all","one",&result,NULL,&flg); CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity one inexplicably missing");
  if (result.i!=5)
    SETERRQ1(PETSC_COMM_SELF,1,"Miscomputed quantity <%d> s/b 5",result.i);

  ierr = HasQuantity
    ((AnaModNumericalProblem)A,"all","two",&flg); CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity two missing");
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"all","two",&result,NULL,&flg); CHKERRQ(ierr);
  if (flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity two strangely present");
  ierr = RetrieveQuantity
    ((AnaModNumericalProblem)A,"all","two",&result,NULL,&flg); CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_SELF,1,"Quantity two inexplicably missing");
  if (result.i!=6)
    SETERRQ1(PETSC_COMM_SELF,1,"Miscomputed quantity <%d> s/b 6",result.i);

  ierr = MatDestroy(&A); CHKERRQ(ierr);
  ierr = AnaModFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  return 0;
}
