#include <stdlib.h>
#include "memory.h"
#include "anamod.h"

/*
 * More simple modules
 */
int IntModule
(AnaModNumericalProblem problem,AnalysisItem *v,int *lv,PetscBool *flg)
{
  PetscFunctionBegin;
  printf("I am computing 5\n");
  v->i = 5;
  PetscFunctionReturn(0);
}

int RealModule
(AnaModNumericalProblem problem,AnalysisItem *v,int *lv,PetscBool *flg)
{
  PetscFunctionBegin;
  printf("I am multiplying %e by 2.2\n",*(double*)problem);
  v->r = *(double*)problem * 2.2;
  PetscFunctionReturn(0);
}

int StringModule
(AnaModNumericalProblem problem,AnalysisItem *v,int *lv,PetscBool *flg)
{
  char *outstring,*instring=*(char**)problem; int l; 
  PetscErrorCode ierr;
  PetscFunctionBegin;
  printf("I am delivering a substring of <%s>\n",instring);
  ierr = PetscMalloc(5*sizeof(char),&outstring); CHKERRQ(ierr);
  memset(outstring,0,5); l = strlen(instring);
  if (l>4) l=4;
  memcpy(outstring,instring,l);
  v->c = outstring;
  PetscFunctionReturn(0);
}

int main(int argc,char **argv)
{
  AnalysisItem result; double rtmp; const char *ctmp;
  AnalysisDataType t;
  PetscBool flg; PetscErrorCode ierr;

  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);

  ierr = RegisterModule
    ("cat1","int1",ANALYSISINTEGER,&IntModule); CHKERRQ(ierr);
  ierr = ComputeQuantity
    (NULL,"cat1","int1",&result,NULL,&flg); CHKERRQ(ierr);
  if (!flg)
    printf("failed to compute int\n");
  else
    printf("computed int %d\n",result.i);

  ierr = RegisterModule
    ("cat1","int2",ANALYSISDOUBLE,&RealModule); CHKERRQ(ierr);
  rtmp = 36.1;
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)&rtmp,"cat1","int2",
     &result,NULL,&flg); CHKERRQ(ierr);
  if (!flg)
    printf("failed to compute int2\n");
  else
    printf("computed real %e\n",result.r);

  ierr = RegisterModule
    ("cat1","int3",ANALYSISSTRING,&StringModule); CHKERRQ(ierr);
  ctmp = "abracadabra";
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)&ctmp,"cat1","int3",
     &result,NULL,&flg); CHKERRQ(ierr);
  if (!flg)
    printf("failed to compute int3\n");
  else
    printf("computed string <%s>\n",result.c);
  ierr = PetscFree(result.c); CHKERRQ(ierr);

  ierr = GetDataType("cat1","int1",&t,NULL); CHKERRQ(ierr);
  if (t!=ANALYSISINTEGER) SETERRQ(PETSC_COMM_SELF,1,"Was expecting int type");
  ierr = GetDataType("cat1","int2",&t,NULL); CHKERRQ(ierr);
  if (t!=ANALYSISDOUBLE) SETERRQ(PETSC_COMM_SELF,1,"Was expecting double type");
  ierr = GetDataType("cat1","int3",&t,NULL); CHKERRQ(ierr);
  if (t!=ANALYSISSTRING) SETERRQ(PETSC_COMM_SELF,1,"Was expecting string type");

  ierr = AnaModFinalize(); CHKERRQ(ierr);

  PetscFinalize();
  return 0;
}
