#include <stdlib.h>
#include "syspro.h"
#include "sysprolinear.h"

/* test for the syspro linear package
   test for problem solving without preprocessors:
   this uses PETSc's KSP for actually solving a linear system
*/

#undef __FUNCT__
#define __FUNCT__ "solvelinear"
static PetscErrorCode solvelinear
(NumericalProblem problem,void *dum,NumericalSolution *rsol)
{
  LinearSystem linear = (LinearSystem)problem;
  KSP solver; Mat A; Vec rhs,sol; PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = LinearSystemGetParts(linear,&A,NULL,&rhs,NULL,NULL); CHKERRQ(ierr);
  ierr = VecDuplicate(rhs,&sol); CHKERRQ(ierr);
  ierr = KSPCreate(MPI_COMM_SELF,&solver); CHKERRQ(ierr);
  ierr = KSPSetOperators(solver,A,A,(MatStructure)0); CHKERRQ(ierr);
  ierr = KSPSolve(solver,rhs,sol); CHKERRQ(ierr);
  ierr = KSPDestroy(&solver); CHKERRQ(ierr);
  *(Vec*)rsol = sol;
  PetscFunctionReturn(0);
}

int main(int argc,char **argv) {
  NumericalProblem problem; NumericalSolution solution;
  Mat A; Vec rhs,sol; PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = SysProInitialize(); CHKERRQ(ierr);
  ierr = SysProDeclareFunctions
    (NULL,NULL,NULL,solvelinear,NULL,NULL,NULL,NULL,NULL,NULL,NULL); CHKERRQ(ierr);
  
  /* set up a simple problem */
  {
    LinearSystem linearproblem;
    int n=10,i; PetscScalar two=2.,mone=-1.,one=1.;

#include "testmat.c"
    /*
     * now create the linear system to contain them
     */
    ierr = CreateLinearSystem(MPI_COMM_SELF,&linearproblem); CHKERRQ(ierr);
    ierr = LinearSystemSetParts
      (linearproblem,A,NULL,rhs,NULL,NULL); CHKERRQ(ierr);
    problem = (NumericalProblem)linearproblem;
  }

  /* solve this problem */
  ierr = PreprocessedProblemSolving(problem,&solution); CHKERRQ(ierr);

  /* check for correctness */
  {
    Vec sol = (Vec)solution,asol; int mone=-1; PetscReal nrm;
    ierr = VecDuplicate(sol,&asol); CHKERRQ(ierr);
    ierr = MatMult(A,sol,asol); CHKERRQ(ierr);
    ierr = VecAXPY(asol,mone,rhs); CHKERRQ(ierr);
    ierr = VecNorm(asol,NORM_2,&nrm); CHKERRQ(ierr);
    if (nrm>1.e-6) {
      SETERRQ1(PETSC_COMM_SELF,1,"Computed wrong solution: norm residual is %e",nrm);
    } else {
      printf("Solution computed to accuracy\n");
    }
    ierr = VecDestroy(&asol); CHKERRQ(ierr);
    ierr = VecDestroy(&sol); CHKERRQ(ierr);
  }

  ierr = MatDestroy(&A); CHKERRQ(ierr);
  ierr = VecDestroy(&rhs); CHKERRQ(ierr);
  ierr = VecDestroy(&sol); CHKERRQ(ierr);
  ierr = PetscFree(problem); CHKERRQ(ierr);
  ierr = SysProFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  PetscFunctionReturn(0);
}
